/*
  Copyright 2023 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.sample.common

import android.annotation.SuppressLint
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.CheckBox
import android.widget.EditText
import android.widget.LinearLayout
import android.widget.Switch
import android.widget.TextView
import androidx.fragment.app.DialogFragment
import com.kakao.sdk.auth.model.Prompt

@SuppressLint("UseSwitchCompatOrMaterialCode")
class ParameterChoiceDialogFragment(
    private val method: String,
    private val settleId: String?,
    val callback: (DialogResult?) -> Unit,
) : DialogFragment() {
    private lateinit var view: View
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View {
        view = inflater.inflate(R.layout.parameter_choice_dialog, container, false)
        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        initDialog()
    }

    private fun initDialog() {
        setTitle()
        setPrompts()
        setLoginHints()
        setScopes()
        setState()
        setSettleId()
        setChannelPublicIds()
        setServiceTerms()

        initButtons()
    }

    private fun setTitle() {
        view.findViewById<TextView>(R.id.title_tv).text = method
    }

    private fun setPrompts() {
        when (method) {
            LOGIN_WITH_KAKAO_TALK -> view.findViewById<LinearLayout>(R.id.prompts_linear_layout).visibility =
                View.GONE

            LOGIN_WITH_NEW_SCOPES -> view.findViewById<LinearLayout>(R.id.prompts_linear_layout).visibility =
                View.GONE

            CERT_LOGIN_WITH_KAKAO_TALK -> view.findViewById<CheckBox>(R.id.prompts_login_checkbox).visibility =
                View.GONE
        }
    }

    private fun setLoginHints() {
        when (method) {
            LOGIN_WITH_KAKAO_TALK -> view.findViewById<LinearLayout>(R.id.login_hints_linear_layout).visibility =
                View.GONE

            LOGIN_WITH_NEW_SCOPES -> view.findViewById<LinearLayout>(R.id.login_hints_linear_layout).visibility =
                View.GONE

            CERT_LOGIN_WITH_KAKAO_TALK -> view.findViewById<LinearLayout>(R.id.login_hints_linear_layout).visibility =
                View.GONE
        }
    }

    private fun setScopes() {
        val scopesSwitch = view.findViewById<Switch>(R.id.scopes_switch)
        scopesSwitch.isChecked = true
        scopesSwitch.isClickable = false
        when (method) {
            LOGIN_WITH_KAKAO_TALK -> view.findViewById<LinearLayout>(R.id.scopes_linear_layout).visibility =
                View.GONE

            LOGIN_WITH_KAKAO_ACCOUNT -> view.findViewById<LinearLayout>(R.id.scopes_linear_layout).visibility =
                View.GONE

            CERT_LOGIN_WITH_KAKAO_TALK -> view.findViewById<LinearLayout>(R.id.scopes_linear_layout).visibility =
                View.GONE

            CERT_LOGIN_WITH_KAKAO_ACCOUNT -> view.findViewById<LinearLayout>(R.id.scopes_linear_layout).visibility =
                View.GONE
        }
    }

    private fun setState() {
        when (method) {
            LOGIN_WITH_KAKAO_TALK -> view.findViewById<LinearLayout>(R.id.state_linear_layout).visibility =
                View.GONE

            LOGIN_WITH_KAKAO_ACCOUNT -> view.findViewById<LinearLayout>(R.id.state_linear_layout).visibility =
                View.GONE

            LOGIN_WITH_NEW_SCOPES -> view.findViewById<LinearLayout>(R.id.state_linear_layout).visibility =
                View.GONE
        }
    }

    private fun setSettleId() {
        view.findViewById<Switch>(R.id.settle_id_switch).apply {
            isChecked = true
            isClickable = false
        }
        val settleIdLayout = view.findViewById<LinearLayout>(R.id.settle_id_linear_layout)
        settleId?.let {
            view.findViewById<EditText>(R.id.settle_id_et).setText(settleId)
        }
        view.findViewById<EditText>(R.id.settle_id_et).isFocusable = false
        when (method) {
            LOGIN_WITH_KAKAO_TALK -> settleIdLayout.visibility = View.GONE
            LOGIN_WITH_KAKAO_ACCOUNT -> settleIdLayout.visibility = View.GONE
            LOGIN_WITH_NEW_SCOPES -> settleIdLayout.visibility = View.GONE
        }
    }

    private fun setChannelPublicIds() {
        val layout = view.findViewById<LinearLayout>(R.id.channel_public_ids_linear_layout)
        when (method) {
            LOGIN_WITH_NEW_SCOPES -> layout.visibility = View.GONE
        }
    }

    private fun setServiceTerms() {
        val layout = view.findViewById<LinearLayout>(R.id.service_terms_linear_layout)
        when (method) {
            LOGIN_WITH_NEW_SCOPES -> layout.visibility = View.GONE
        }
    }

    private fun initButtons() {
        view.findViewById<Button>(R.id.cancel_btn).setOnClickListener {
            dismiss()
            callback(null)
        }
        view.findViewById<Button>(R.id.request_btn).setOnClickListener {
            dismiss()
            callback(getDialogResult())
        }
    }

    private fun getDialogResult(): DialogResult {
        val prompts = mutableListOf<Prompt>()
        var loginHints: String? = null
        val scopes = mutableListOf<String>()
        var state: String? = null
        var nonce: String? = null
        var settleId: String? = null
        val channelPublicIds = mutableListOf<String>()
        val serviceTerms = mutableListOf<String>()

        if (view.findViewById<Switch>(R.id.prompts_switch).isChecked) {
            if (view.findViewById<CheckBox>(R.id.prompts_login_checkbox).isChecked) {
                prompts.add(Prompt.LOGIN)
            }
            if (view.findViewById<CheckBox>(R.id.prompts_create_checkbox).isChecked) {
                prompts.add(Prompt.CREATE)
            }
            if (view.findViewById<CheckBox>(R.id.prompts_select_account_checkbox).isChecked) {
                prompts.add(Prompt.SELECT_ACCOUNT)
            }
        }
        if (view.findViewById<Switch>(R.id.login_hint_switch).isChecked) {
            loginHints = view.findViewById<EditText>(R.id.login_hint_et).text.toString()
        }
        if (view.findViewById<Switch>(R.id.scopes_switch).isChecked) {
            scopes.addAll(view.findViewById<EditText>(R.id.scopes_et).text.toString().split(","))
        }
        if (view.findViewById<Switch>(R.id.state_switch).isChecked) {
            state = view.findViewById<EditText>(R.id.state_et).text.toString()
        }
        if (view.findViewById<Switch>(R.id.nonce_switch).isChecked) {
            nonce = view.findViewById<EditText>(R.id.nonce_et).text.toString()
        }
        if (view.findViewById<Switch>(R.id.settle_id_switch).isChecked) {
            settleId = view.findViewById<EditText>(R.id.settle_id_et).text.toString()
        }
        if (view.findViewById<Switch>(R.id.channel_public_ids_switch).isChecked) {
            channelPublicIds.addAll(
                view.findViewById<EditText>(R.id.channel_public_ids_et).text.toString().split(",")
            )
        }
        if (view.findViewById<Switch>(R.id.service_terms_switch).isChecked) {
            serviceTerms.addAll(
                view.findViewById<EditText>(R.id.service_terms_et).text.toString().split(",")
            )
        }
        return DialogResult(
            prompts = prompts,
            loginHints = loginHints,
            scopes = scopes,
            state = state,
            nonce = nonce,
            settleId = settleId,
            channelPublicIds = channelPublicIds,
            serviceTerms = serviceTerms,
        )
    }

    companion object {
        const val LOGIN_WITH_KAKAO_TALK = "loginWithKakaoTalk"
        const val LOGIN_WITH_KAKAO_ACCOUNT = "loginWithKakaoAccount"
        const val LOGIN_WITH_NEW_SCOPES = "loginWithNewScopes"
        const val CERT_LOGIN_WITH_KAKAO_TALK = "certLoginWithKakaoTalk"
        const val CERT_LOGIN_WITH_KAKAO_ACCOUNT = "certLoginWithKakaoAccount"
    }
}
